==> 0.2.1.tar.gz: lib/api/ApiController2.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:intl/intl.dart';
import 'package:snosey_flutter_package/Enums.dart';
import 'package:snosey_flutter_package/api/ApiResponse.dart';

class ApiController {
  static late String baseUrl;
  static late String appSecret;
  static late String apiKey;
  static late String userAgent;
  static late String serverErrorMessage;
  static late String refreshTokenUrl;
  static Map<String, String> staticParams = {};
  static String _cookies = "";
  final String apiUrl;
  final RequestTypeEnum requestType;
  static Map<String, String> staticHeaders = {};
  late Response response;

  Map<RequestTypeEnum, String> requestTypeMap = {
    RequestTypeEnum.PUT: "PUT",
    RequestTypeEnum.POST: "POST",
    RequestTypeEnum.GET: "GET",
    RequestTypeEnum.DELETE: "DELETE"
  };

  ApiController(this.apiUrl, this.requestType);

  static init({
    required String baseUrl,
    required String serverErrorMessage,
    required String appSecret,
    required UserAgentEnum userAgent,
    required String apiKey,
    required String refreshTokenUrl,
  }) {
    ApiController.baseUrl = baseUrl;
    ApiController.refreshTokenUrl = refreshTokenUrl;
    ApiController.serverErrorMessage = serverErrorMessage;
    _addCookies("Secret", appSecret);
    _addCookies("Api-Key", apiKey);
    staticHeaders["User-Agent"] =
        userAgent.toString().replaceFirst("UserAgentEnum.", "");
  }

  static _addCookies(String key, String value) {
    _cookies += "$key=$value;";
  }

  Future<ApiResponse> sendRequest(
      {Map<String, dynamic>? body,
      Map<String, File>? files,
      bool isMultiPart = false,
      Map<String, String> header = const {},
      Map<String, String> queryParameters = const {}}) async {
    staticParams.addAll(queryParameters);
    staticHeaders["Cookie"] = _cookies;
    staticHeaders.addAll(header);
    /*var uri = Uri.https(baseUrl, apiUrl, staticParams);
    log('request url: ${uri.toString()}');
    */
    log('request header: ${staticHeaders.toString()}');
    var api = Dio(
      BaseOptions(
        baseUrl: baseUrl,
        queryParameters: staticParams,
        headers: staticHeaders,
        contentType: 'application/json; charset=UTF-8',
        responseType: ResponseType.json,
        method: requestType.toString().replaceAll("RequestTypeEnum.", ""),
      ),
    );
    var response = await api.request(
      apiUrl,
    );
    if (response.statusCode == HttpStatus.ok) {
      if (response.headers.value("authorization") != null) {
        Auth._setBearerToken(
            token: response.headers.value("authorization")!,
            expiration: response.headers.value("expires")!);
        Auth._setRefreshToken(response.headers.value("set-cookie")!);
      }

      var apiResponse = ApiResponse();
      apiResponse.setStatus(jsonDecode(response.headers.value("x-status")!));
      if (response.headers.value("x-pagination")!=null) {
        apiResponse.pagination.hasPagination = true;
        apiResponse.pagination
            .set(jsonDecode(response.headers.value("x-pagination")!));
      } else {
        apiResponse.pagination.hasPagination = false;
      }
      log('response status: ${response.headers["x-status"]}');
      if (apiResponse.success) {
        apiResponse.objectResponse = jsonDecode(response.data);
        return apiResponse;
      } else
        throw utf8.decode(base64.decode(apiResponse.errorMessage));
    } else {
      throw serverErrorMessage;
    }
  }

}

class Auth {
  static var expirationFormat = DateFormat("E, dd MMM yyy HH:mm:ss 'GMT'");
  static Timer? reNewExpirationTimer;

  static _setBearerToken({
    required String token,
    required String expiration,
  }) {
    ApiController.staticHeaders["Authorization"] = token;
    var expirationMin = DateTime.now()
            .difference(expirationFormat.parse(expiration))
            .inMinutes -
        5;
    print("renew in $expirationMin min");
    reNewExpirationTimer = Timer(Duration(minutes: expirationMin), () {
      reNewToken();
    });
  }

  static Future<bool> reNewToken() async {
    final storage = new FlutterSecureStorage();
    String? expires = await storage.read(key: "expires");
    if (expires != null &&
        expirationFormat.parse(expires).isAfter(DateTime.now())) {
      String? token = await storage.read(key: "refreshToken");
      var response = await ApiController(
              ApiController.refreshTokenUrl, RequestTypeEnum.POST)
          .sendRequest(body: {"Token": token!});
      return response.success;
    } else {
      return false;
    }
  }

  static clearToken() {
    if (reNewExpirationTimer != null) reNewExpirationTimer!.cancel();
    if (ApiController.staticHeaders.containsKey("Authorization"))
      ApiController.staticHeaders.remove("Authorization");
    final storage = new FlutterSecureStorage();
    storage.deleteAll();
  }

  static _setRefreshToken(String cookies) {
    final storage = new FlutterSecureStorage();
    cookies.split(";").forEach((element) {
      var key = element.split("=").first;
      var value = element.split("=").last;
      if (key == "refreshToken" || key == "expires") {
        storage.write(key: key, value: value);
      }
    });
  }
}

==> 0.2.2.tar.gz: lib/api/ApiController2.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:intl/intl.dart';
import 'package:snosey_flutter_package/Enums.dart';
import 'package:snosey_flutter_package/api/ApiResponse.dart';

class ApiController {
  static late String baseUrl;
  static late String appSecret;
  static late String apiKey;
  static late String userAgent;
  static late String serverErrorMessage;
  static late String refreshTokenUrl;
  static Map<String, String> staticParams = {};
  static String _cookies = "";
  final String apiUrl;
  final RequestTypeEnum requestType;
  static Map<String, String> staticHeaders = {};
  late Response response;

  Map<RequestTypeEnum, String> requestTypeMap = {
    RequestTypeEnum.PUT: "PUT",
    RequestTypeEnum.POST: "POST",
    RequestTypeEnum.GET: "GET",
    RequestTypeEnum.DELETE: "DELETE"
  };

  ApiController(this.apiUrl, this.requestType);

  static init({
    required String baseUrl,
    required String serverErrorMessage,
    required String appSecret,
    required UserAgentEnum userAgent,
    required String apiKey,
    required String refreshTokenUrl,
  }) {
    ApiController.baseUrl = baseUrl;
    ApiController.refreshTokenUrl = refreshTokenUrl;
    ApiController.serverErrorMessage = serverErrorMessage;
    _addCookies("Secret", appSecret);
    _addCookies("Api-Key", apiKey);
    staticHeaders["User-Agent"] =
        userAgent.toString().replaceFirst("UserAgentEnum.", "");
  }

  static _addCookies(String key, String value) {
    _cookies += "$key=$value;";
  }

  Future<ApiResponse> sendRequest(
      {Map<String, dynamic>? body,
      Map<String, File>? files,
      bool isMultiPart = false,
      Map<String, String> header = const {},
      Map<String, String> queryParameters = const {}}) async {
    staticParams.addAll(queryParameters);
    staticHeaders["Cookie"] = _cookies;
    staticHeaders.addAll(header);
    /*var uri = Uri.https(baseUrl, apiUrl, staticParams);
    log('request url: ${uri.toString()}');
    */
    log('request header: ${staticHeaders.toString()}');

    var response = await Dio(
      BaseOptions(
        baseUrl: baseUrl,
        contentType: 'application/json; charset=UTF-8',
        responseType: ResponseType.json,
        method: requestType.toString().replaceAll("RequestTypeEnum.", ""),
      ),
    ).request(
      apiUrl,
      data: body,
      queryParameters: staticParams,
    );

    if (response.statusCode == HttpStatus.ok) {
      if (response.headers.value("authorization") != null) {
        Auth._setBearerToken(
            token: response.headers.value("authorization")!,
            expiration: response.headers.value("expires")!);
        Auth._setRefreshToken(response.headers.value("set-cookie")!);
      }

      var apiResponse = ApiResponse();
      apiResponse.setStatus(jsonDecode(response.headers.value("x-status")!));
      if (response.headers.value("x-pagination") != null) {
        apiResponse.pagination.hasPagination = true;
        apiResponse.pagination
            .set(jsonDecode(response.headers.value("x-pagination")!));
      } else {
        apiResponse.pagination.hasPagination = false;
      }
      log('response status: ${response.headers["x-status"]}');
      if (apiResponse.success) {
        apiResponse.objectResponse = jsonDecode(response.data);
        return apiResponse;
      } else
        throw utf8.decode(base64.decode(apiResponse.errorMessage));
    } else {
      throw serverErrorMessage;
    }
  }
}

class Auth {
  static var expirationFormat = DateFormat("E, dd MMM yyy HH:mm:ss 'GMT'");
  static Timer? reNewExpirationTimer;

  static _setBearerToken({
    required String token,
    required String expiration,
  }) {
    ApiController.staticHeaders["Authorization"] = token;
    var expirationMin = DateTime.now()
            .difference(expirationFormat.parse(expiration))
            .inMinutes -
        5;
    print("renew in $expirationMin min");
    reNewExpirationTimer = Timer(Duration(minutes: expirationMin), () {
      reNewToken();
    });
  }

  static Future<bool> reNewToken() async {
    final storage = new FlutterSecureStorage();
    String? expires = await storage.read(key: "expires");
    if (expires != null &&
        expirationFormat.parse(expires).isAfter(DateTime.now())) {
      String? token = await storage.read(key: "refreshToken");
      var response = await ApiController(
              ApiController.refreshTokenUrl, RequestTypeEnum.POST)
          .sendRequest(body: {"Token": token!});
      return response.success;
    } else {
      return false;
    }
  }

  static clearToken() {
    if (reNewExpirationTimer != null) reNewExpirationTimer!.cancel();
    if (ApiController.staticHeaders.containsKey("Authorization"))
      ApiController.staticHeaders.remove("Authorization");
    final storage = new FlutterSecureStorage();
    storage.deleteAll();
  }

  static _setRefreshToken(String cookies) {
    final storage = new FlutterSecureStorage();
    cookies.split(";").forEach((element) {
      var key = element.split("=").first;
      var value = element.split("=").last;
      if (key == "refreshToken" || key == "expires") {
        storage.write(key: key, value: value);
      }
    });
  }
}
